<?php

declare(strict_types=1);

namespace FiloBlu\RuleDayFromBirthday\Cron;

use Exception;
use FiloBlu\RuleDayFromBirthday\Helper\Data as Helper;
use FiloBlu\RuleDayFromBirthday\Controller\Adminhtml\Promo\Quote\Birthdaycoupon;
use Magento\Framework\Event\ManagerInterface;
use Magento\Framework\Exception\InputException;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Stdlib\DateTime\DateTime;
use RuntimeException;

/**
 * @class SendBulkSubscriber
 * @package FiloBlu\BirthdayCoupon\Cron
 */
class SendBulkSubscriber
{

    /**
     * @var Helper
     */
    private $helper;
    /**
     * @var Birthdaycoupon
     */
    private $birthdaycouponGenerator;
    /**
     * @var DateTime
     */
    private $dateTime;
    /**
     * @var ManagerInterface
     */
    private $eventManager;

    /**
     * @param Helper $helper
     * @param Birthdaycoupon $birthdaycouponGenerator
     * @param ManagerInterface $eventManager
     * @param DateTime $dateTime
     */
    public function __construct(
        Helper           $helper,
        Birthdaycoupon   $birthdaycouponGenerator,
        ManagerInterface $eventManager,
        DateTime         $dateTime
    ) {
        $this->helper = $helper;
        $this->birthdaycouponGenerator = $birthdaycouponGenerator;
        $this->eventManager = $eventManager;
        $this->dateTime = $dateTime;
    }

    /**
     * @throws LocalizedException
     * @throws InputException
     * @throws Exception
     */
    public function sendBulkSubscriber(): SendBulkSubscriber
    {
        $daysOffset = $this->helper->getCouponCreationDaysOffset();
        $birthdayDate = $this->dateTime->date('Y-m-d H:i:s', strtotime("+ $daysOffset days"));

        $subscribers = $this->helper->getSubscribersForDate($birthdayDate);
        if (!$subscribers) {
            return $this;
        }

        $storeSubscribers = [];
        foreach ($subscribers as $subscriber) {
            $storeSubscribers[(int) $subscriber['store_id']][] = $subscriber;
        }


        $bulkData = $coupons = [];
        foreach ($storeSubscribers as $storeId => $subscribers) {
            if (!$rule = $this->helper->getBirthdayCouponRule($storeId)) {
                unset($storeSubscribers[$storeId]);
                continue;
            }

            $couponQuantity = $this->countArray($subscribers);
            $coupons[$storeId] = $this->helper->generateCoupon($rule, $couponQuantity);
        }

        foreach ($storeSubscribers as $storeId => $subscribers) {
            if (!array_key_exists($storeId, $coupons)) {
                continue;
            }

            if (count($subscribers) > count($coupons[$storeId])) {
                throw new RuntimeException(__("Not enough coupons were created for store Id $storeId's subscribers"));
            }

            foreach ($subscribers as $subscriber) {
                $bulkData[] = [
                    'subscriber' => $subscriber,
                    'coupon' => array_pop($coupons[$storeId])
                ];
            }

            $chunks = array_chunk($bulkData, $this->helper->maxSubscriberForBatchInBulk());
            foreach ($chunks as $chunk) {
                $this->eventManager->dispatch(
                    'filoblu_birthday_coupon_generate_after',
                    [
                        'bulkData' => $chunk
                    ]
                );
            }
        }

        return $this;
    }

    /**
     * @param array $array
     * @return int
     */
    private function countArray(array $array): int
    {
        if (!is_countable($array)) {
            return 0;
        }

        return count($array);
    }
}
