<?php

declare(strict_types=1);

namespace FiloBlu\RuleDayFromBirthday\Observer;

use FiloBlu\RuleDayFromBirthday\Api\CustomCouponRepositoryInterface;
use FiloBlu\RuleDayFromBirthday\Api\Data\CustomCouponInterface;
use Magento\Backend\Block\Widget;
use Magento\Framework\Data\Form;
use Magento\Framework\Data\Form\Element\Fieldset;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Registry;
use Magento\Framework\UrlInterface;
use Magento\SalesRule\Model\RegistryConstants;

/**
 * @class AddGenerateBirthdayCouponObserver
 * @package FiloBlu\RuleDayFromBirthday\Observer
 */
class AddGenerateBirthdayCouponObserver implements ObserverInterface
{
    protected $urlBuilder;

    protected $widget;

    /**
     * @var Registry
     */
    private $registry;
    /**
     * @var CustomCouponRepositoryInterface
     */
    private $customCouponRepository;

    /**
     * @param UrlInterface $urlBuilder
     * @param Widget $widget
     * @param Registry $registry
     * @param CustomCouponRepositoryInterface $customCouponRepository
     */
    public function __construct(
        UrlInterface  $urlBuilder,
        Widget        $widget,
        Registry      $registry,
        CustomCouponRepositoryInterface $customCouponRepository
    ) {
        $this->urlBuilder = $urlBuilder;
        $this->widget = $widget;
        $this->registry = $registry;
        $this->customCouponRepository = $customCouponRepository;
    }

    /**
     * Execute observer.
     * @param Observer $observer
     * @return void
     */
    public function execute(Observer $observer)
    {
        /** @var Form $form */
        $form = $observer->getForm();

        $model = $this->registry->registry(RegistryConstants::CURRENT_SALES_RULE);
        try {
            /** @var CustomCouponInterface $customCoupon */
            $customCoupon = $this->customCouponRepository->getByKey($model->getRuleId(), 'rule_id');
            $data['length'] = $customCoupon->getLength();
            $data['format'] = $customCoupon->getFormat();
            $data['prefix'] = $customCoupon->getPrefix();
            $data['suffix'] = $customCoupon->getSuffix();
            $data['dash'] = $customCoupon->getDash();
            $form->setValues($data);
        } catch (NoSuchEntityException $exception) {
        }

        /** @var Fieldset $informationFieldset */
        $informationFieldset = $form->getElement('information_fieldset');

        $expireDate = $informationFieldset->addField(
            'filoblu_coupon_expire_date',
            'text',
            [
                'name' => 'expiration_days',
                'label' => __('Coupon Expiration Days')
            ],
            'dash'
        );

        $expireDate->setValue($model->getDataByPath('extension_attributes/filoblu_sales_rule_coupon/expiration_days'));

        $birthdayCoupon = $informationFieldset->addField(
            'filoblu_birthday_coupon',
            'select',
            [
                'name' => 'is_birthday_coupon',
                'label' => __('Is Birthday Coupon'),
                'values' => [
                    '1' => 'Yes',
                    '0' => 'No'
                ]
            ],
            'filoblu_coupon_expire_date'
        );

        $couponType = $model->getDataByPath('extension_attributes/filoblu_sales_rule_coupon/type');
        $isBirthdayCoupon = $couponType === CustomCouponInterface::BIRTHDAY_COUPON_TYPE;
        $birthdayCoupon->setValue((int) $isBirthdayCoupon);

        $fieldset = $form->addFieldset('information_fieldset_birthday', []);

        $generateUrl = $this->urlBuilder->getUrl('sales_rule/*/birthdaycoupon');
        $fieldset->addField(
            'generate_birthday_coupon_button',
            'note',
            [
                'text' => $this->widget->getButtonHtml(
                    __('Generate Birthday Coupon'),
                    "generateCouponCodes('coupons_' ,'{$generateUrl}', 'couponCodesGridJsObject')",
                    'generate'
                )
            ]
        );
    }
}
